//************************************************************************** 
//* Export.cpp	- MAX2MY3D File Exporter
//* 
//* This tool was created by Zhuck Dmitry (ZDimitor).
//* Everyone can use it as wants ( i'll be happy if it helps to someone :) ).
//*
//***************************************************************************
#include "MAX2MY3DExp.h"  
#include <direct.h> 

#include <bmmlib.h> 
#include <bitmap.h> 

//**********************************************************************
//                      some usefull functions
//**********************************************************************
#include "../my3d_common/somefuncs.hpp"

using namespace irr;
using namespace core;

//-------------------------------------------------------------------
void MAX2MY3D::DumpMaterial(Mtl* mtl, int mtlID, int subNo, int indentLevel)
{
	int i;
	TimeValue t = GetStaticFrame();
	
	if (!mtl) return;
	
	TSTR indent = GetIndent(indentLevel+1);
	
	TSTR className;
	mtl->GetClassName(className);

	if (subNo == -1) 
    {	// Top level material
		fprintf(pStream,"%s%s %d {\n",indent.data(), ID_MATERIAL, mtlID);
	}
	else 
    {	fprintf(pStream,"%s%s %d {\n",indent.data(), ID_SUBMATERIAL, subNo);
	}

	fprintf(pStream,"%s\t%s \"%s\"\n",indent.data(), ID_MATNAME, FixupName(mtl->GetName()));
	fprintf(pStream,"%s\t%s \"%s\"\n",indent.data(), ID_MATCLASS, FixupName(className));

	// We know the Standard material, so we can get some extra info
	if (mtl->ClassID() == Class_ID(DMTL_CLASS_ID, 0)) 
    {
		StdMat* std = (StdMat*)mtl;

		fprintf(pStream,"%s\t%s %s\n",indent.data(), ID_AMBIENT, Format(std->GetAmbient(t)));
		fprintf(pStream,"%s\t%s %s\n",indent.data(), ID_DIFFUSE, Format(std->GetDiffuse(t)));
		fprintf(pStream,"%s\t%s %s\n",indent.data(), ID_SPECULAR, Format(std->GetSpecular(t)));
		fprintf(pStream,"%s\t%s %s\n",indent.data(), ID_SHINE, Format(std->GetShininess(t)));
		fprintf(pStream,"%s\t%s %s\n",indent.data(), ID_SHINE_STRENGTH, Format(std->GetShinStr(t)));
		fprintf(pStream,"%s\t%s %s\n",indent.data(), ID_TRANSPARENCY, Format(std->GetXParency(t)));
		fprintf(pStream,"%s\t%s %s\n",indent.data(), ID_WIRESIZE, Format(std->WireSize(t)));

		fprintf(pStream,"%s\t%s ", indent.data(), ID_SHADING);
		switch(std->GetShading()) 
        {
		case SHADE_CONST: fprintf(pStream,"%s\n", ID_MAT_SHADE_CONST); break;
		case SHADE_PHONG: fprintf(pStream,"%s\n", ID_MAT_SHADE_PHONG); break;
		case SHADE_METAL: fprintf(pStream,"%s\n", ID_MAT_SHADE_METAL); break;
		case SHADE_BLINN: fprintf(pStream,"%s\n", ID_MAT_SHADE_BLINN); break;
		default: fprintf(pStream,"%s\n", ID_MAT_SHADE_OTHER); break;
		}
		
		fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_XP_FALLOFF, Format(std->GetOpacFalloff(t)));
		fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_SELFILLUM, Format(std->GetSelfIllum(t)));
		
		if (std->GetTwoSided()) 
        {   fprintf(pStream,"%s\t%s\n", indent.data(), ID_TWOSIDED);
		}		
		if (std->GetWire()) 
        {   fprintf(pStream,"%s\t%s\n", indent.data(), ID_WIRE);
		}		
		if (std->GetWireUnits()) 
        {	fprintf(pStream,"%s\t%s\n", indent.data(), ID_WIREUNITS);
		}		
		fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_FALLOFF, std->GetFalloffOut() ? ID_FALLOFF_OUT : ID_FALLOFF_IN);
		
		if (std->GetFaceMap()) 
        {	fprintf(pStream,"%s\t%s\n", indent.data(), ID_FACEMAP);
		}		
		if (std->GetSoften()) 
        {	fprintf(pStream,"%s\t%s\n", indent.data(), ID_SOFTEN);
		}
		
		fprintf(pStream,"%s\t%s ", indent.data(), ID_XP_TYPE);
		switch (std->GetTransparencyType()) {
		case TRANSP_FILTER: fprintf(pStream,"%s\n", ID_MAP_XPTYPE_FLT); break;
		case TRANSP_SUBTRACTIVE: fprintf(pStream,"%s\n", ID_MAP_XPTYPE_SUB); break;
		case TRANSP_ADDITIVE: fprintf(pStream,"%s\n", ID_MAP_XPTYPE_ADD); break;
		default: fprintf(pStream,"%s\n", ID_MAP_XPTYPE_OTH); break;
		}
	}
	else 
    {
		// Note about material colors:
		// This is only the color used by the interactive renderer in MAX.
		// To get the color used by the scanline renderer, we need to evaluate
		// the material using the mtl->Shade() method.
		// Since the materials are procedural there is no real "diffuse" color for a MAX material
		// but we can at least take the interactive color.
		
		fprintf(pStream,"%s\t%s %s\n",indent.data(), ID_AMBIENT, Format(mtl->GetAmbient()));
		fprintf(pStream,"%s\t%s %s\n",indent.data(), ID_DIFFUSE, Format(mtl->GetDiffuse()));
		fprintf(pStream,"%s\t%s %s\n",indent.data(), ID_SPECULAR, Format(mtl->GetSpecular()));
		fprintf(pStream,"%s\t%s %s\n",indent.data(), ID_SHINE, Format(mtl->GetShininess()));
		fprintf(pStream,"%s\t%s %s\n",indent.data(), ID_SHINE_STRENGTH, Format(mtl->GetShinStr()));
		fprintf(pStream,"%s\t%s %s\n",indent.data(), ID_TRANSPARENCY, Format(mtl->GetXParency()));
		fprintf(pStream,"%s\t%s %s\n",indent.data(), ID_WIRESIZE, Format(mtl->WireSize()));
	}    

    for (i=0; i<mtl->NumSubTexmaps(); i++) 
    {
		Texmap* subTex = mtl->GetSubTexmap(i);
		float amt = 1.0f;
		if (subTex) 
        {	// If it is a standard material we can see if the map is enabled.
			if (mtl->ClassID() == Class_ID(DMTL_CLASS_ID, 0)) 
            {	if (!((StdMat*)mtl)->MapEnabled(i))
					continue;
				amt = ((StdMat*)mtl)->GetTexmapAmt(i, 0);				
			}
			DumpTexture(subTex, mtl->ClassID(), i, amt, indentLevel+1);
		}
	}	
    
	if (mtl->NumSubMtls() > 0)  
    {	fprintf(pStream,"%s\t%s %d\n",indent.data(), ID_NUMSUBMTLS, mtl->NumSubMtls());
		
		for (i=0; i<mtl->NumSubMtls(); i++) 
        {	Mtl* subMtl = mtl->GetSubMtl(i);
			if (subMtl) 
            {	DumpMaterial(subMtl, 0, i, indentLevel+1);
			}
		}
	}

	fprintf(pStream,"%s}\n", indent.data());
}
//-------------------------------------------------------------------
void MAX2MY3D::DumpTexture(Texmap* tex, Class_ID cid, int subNo, float amt, int indentLevel)
{
	if (!tex) return;
	
	TSTR indent = GetIndent(indentLevel+1);
	
	TSTR className;
	tex->GetClassName(className);
	
	fprintf(pStream,"%s%s {\n", indent.data(), GetMapID(cid, subNo));	
	fprintf(pStream,"%s\t%s \"%s\"\n", indent.data(), ID_TEXNAME, FixupName(tex->GetName()));
	fprintf(pStream,"%s\t%s \"%s\"\n", indent.data(), ID_TEXCLASS, FixupName(className));
	
	
	// If we include the subtexture ID, a parser could be smart enough to get
	// the class name of the parent texture/material and make it mean something.
	fprintf(pStream,"%s\t%s %d\n", indent.data(), ID_TEXSUBNO, subNo);	
	fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_TEXAMOUNT, Format(amt));
	
	// Is this a bitmap texture?
	// We know some extra bits 'n pieces about the bitmap texture
	if (tex->ClassID() == Class_ID(BMTEX_CLASS_ID, 0x00)) 
    {
		TSTR mapName = ((BitmapTex *)tex)->GetMapName();
		fprintf(pStream,"%s\t%s \"%s\"\n", indent.data(), ID_BITMAP, mapName);//FixupName(mapName));
		
		//--------------------------------------------------------
		
        TSTR texName = myCutName(mapName);  
        char textureName[256];	
		
        memset(textureName, 0, 256);
        if (texName.Length()<256)
            sprintf (textureName, "%s", texName);
        else
        {   for (int i=0 ;i<255; i++)
                textureName[i] = texName[i];    
            textureName[255]=0;
        } 		
        
		// checking - is this lightmap or not ?

        char *lightMapsStr = "lightingmap\0";

        int lenLigStr = strlen(lightMapsStr);
        int lenName = strlen(textureName);
        bool lightmap = false;
		for (int c=0; c<lenName && lenLigStr<lenName; c++)
		{	if (textureName[c]=='.' && c>=lenLigStr &&
                (textureName[c-11]=='l' || textureName[c-11]=='L') && 
                (textureName[c-10]=='i' || textureName[c-10]=='I') &&
                (textureName[c-9] =='g' || textureName[c-9] =='G') && 
                (textureName[c-8] =='h' || textureName[c-8] =='H') &&
                (textureName[c-7] =='t' || textureName[c-7] =='T') && 
                (textureName[c-6] =='i' || textureName[c-6] =='I') &&
                (textureName[c-5] =='n' || textureName[c-5] =='N') && 
                (textureName[c-4] =='g' || textureName[c-4] =='G') &&
                (textureName[c-3] =='m' || textureName[c-3] =='M') && 
                (textureName[c-2] =='a' || textureName[c-2] =='A') &&
                (textureName[c-1] =='p' || textureName[c-1] =='P')
                ) 
			{	if ((c+1)<(256-4))
				{   lightmap = true;
					break;
				}
			}
		}		

        u16 id;

        Bitmap* map = ((BitmapTex *)tex)->GetBitmap(0);

        if (!lightmap || !map || (lightmap && !bEmbeddedLightmap))
        {
            id = MY_TEX_FNAME_ID;
            myWriteToFile(&id, sizeof(id));    
            myWriteToFile(&textureName, 256);

			if (lightmap)
			{
				// copy lightmap to the Lightmaps directory
				char cmd[4096];
				sprintf(cmd, "copy %s Lightmaps/", mapName);
				system(cmd);
			}

        }else
        {
            id = MY_TEXDATA_ID;
            myWriteToFile(&id, sizeof(id));    

            id = MY_TEXDATA_HEADER_ID;
            myWriteToFile(&id, sizeof(id));               

            int width  = map->Width();
            int height = map->Height();

            SMyTexDataHeader texDataHeader;

            for (int i=0 ;i<255; i++)
                texDataHeader.Name[i] = textureName[i];    
            texDataHeader.Name[255]=0;  
            
            if (width>1024) width=1024;
            if (height>768) height=768;
   
            texDataHeader.Width = width;
            texDataHeader.Height= height;

			if (eLightmapQuality == lq16bit)
				texDataHeader.PixelFormat = MY_PIXEL_FORMAT_16;
			else
				texDataHeader.PixelFormat = MY_PIXEL_FORMAT_24;

			if (eLightmapCompression == lcSimple)
				texDataHeader.ComprMode  = MY_TEXDATA_COMPR_SIMPLE_ID;
			else if (eLightmapCompression == lcRLE)
				texDataHeader.ComprMode  = MY_TEXDATA_COMPR_RLE_ID;
			else				
				texDataHeader.ComprMode  = MY_TEXDATA_COMPR_NONE_ID;            

            myWriteToFile(&texDataHeader, sizeof(SMyTexDataHeader));
           
            static BMM_Color_64 pix_ptr[1024];

            f32 WordToByte = 255./65535.; 
            f32 WordTo5Bit = 31./65535.; 

            WORD last_r, last_g, last_b;
            u32 nRepeat=0;

            SMyPixelColor24 pixColor24(0,0,0);
            SMyPixelColor16 pixColor16(0,0,0);

			void* buffer = 0;
			void* buffer_compressed = 0;
			int nBytesAllocated=0;

			if (texDataHeader.ComprMode == MY_TEXDATA_COMPR_NONE_ID||
				texDataHeader.ComprMode == MY_TEXDATA_COMPR_RLE_ID
				)
			{	// allocate buffer
				if (texDataHeader.PixelFormat==MY_PIXEL_FORMAT_24)
				{	nBytesAllocated = width*height*sizeof(SMyPixelColor24);
					buffer = (void*) new SMyPixelColor24[width*height];
					buffer_compressed = (void*) new SMyPixelColor24[width*height];
				}
				else
				{	nBytesAllocated = width*height*sizeof(SMyPixelColor16);
					buffer = (void*) new SMyPixelColor16[width*height];
					buffer_compressed = (void*) new SMyPixelColor24[width*height];
				}				
			}
                      
            for (int y=0; y<height; y++)
            {
                map->GetLinearPixels(0, y, width, (BMM_Color_64*)&pix_ptr);

                if (texDataHeader.ComprMode == MY_TEXDATA_COMPR_SIMPLE_ID)
                // simple compression
                {
					// write data to file
                    for (int x=0; x<width; x++)
                    {
                        if (texDataHeader.PixelFormat==MY_PIXEL_FORMAT_24)
                        {   pix_ptr[x].r*= WordToByte; pix_ptr[x].g*= WordToByte;
                            pix_ptr[x].b*= WordToByte;  

                            if (x==0&&y==0)
                            {   last_r = pix_ptr[x].r; last_g = pix_ptr[x].g;
                                last_b = pix_ptr[x].b;
                            }

                            pixColor24 = SMyPixelColor24(last_r, last_g, last_b);

                            if ((pix_ptr[x].r!=last_r || pix_ptr[x].g!=last_g || 
                                 pix_ptr[x].b!=last_b
                                 )
                                )
                            {                                
                                myWriteToFile(&nRepeat, sizeof(nRepeat)); 
                                myWriteToFile(&pixColor24, sizeof(SMyPixelColor24));
                                
                                nRepeat =1;

                                if (x==(width-1)&&y==(height-1))
                                {   pixColor24 = SMyPixelColor24(pix_ptr[x].r, pix_ptr[x].g, pix_ptr[x].b);
                                    
                                    myWriteToFile(&nRepeat, sizeof(nRepeat)); 
                                    myWriteToFile(&pixColor24, sizeof(SMyPixelColor24));
                                }
                            }
                            else
                            {   nRepeat++;                                               

                                if (x==(width-1)&&y==(height-1))
                                {   myWriteToFile(&nRepeat, sizeof(nRepeat)); 
                                    myWriteToFile(&pixColor24, sizeof(SMyPixelColor24));
                                }
                            }
                        }else
                        {
                            pix_ptr[x].r*= WordTo5Bit; pix_ptr[x].g*= WordTo5Bit;
                            pix_ptr[x].b*= WordTo5Bit; 
                            
                            if (x==0&&y==0)
                            {   last_r = pix_ptr[x].r; last_g = pix_ptr[x].g;
                                last_b = pix_ptr[x].b;
                            }

                            pixColor16 = SMyPixelColor16(last_r, last_g, last_b);

                            if ((pix_ptr[x].r!=last_r || pix_ptr[x].g!=last_g || 
                                 pix_ptr[x].b!=last_b
                                 )
                                )
                            {                                
                                myWriteToFile(&nRepeat, sizeof(nRepeat)); 
                                myWriteToFile(&pixColor16, sizeof(SMyPixelColor16));
                                
                                nRepeat =1;

                                if (x==(width-1)&&y==(height-1))
                                {   pixColor16 = SMyPixelColor16(pix_ptr[x].r, pix_ptr[x].g, pix_ptr[x].b);
                                    
                                    myWriteToFile(&nRepeat, sizeof(nRepeat)); 
                                    myWriteToFile(&pixColor16, sizeof(SMyPixelColor16));
                                }
                            }
                            else
                            {   nRepeat++;                                               

                                if (x==(width-1)&&y==(height-1))
                                {   myWriteToFile(&nRepeat, sizeof(nRepeat)); 
                                    myWriteToFile(&pixColor16, sizeof(SMyPixelColor16));
                                }
                            }
                        }                        

                        last_r = pix_ptr[x].r;
                        last_g = pix_ptr[x].g;
                        last_b = pix_ptr[x].b;
                    }
                }
				else
				// RLE compression or None compression
				{
					// fill the buffer with data
					for (int x=0; x<width; x++)
			        {
						if (texDataHeader.PixelFormat==MY_PIXEL_FORMAT_24)
					    {   pix_ptr[x].r*= WordToByte; pix_ptr[x].g*= WordToByte;
						    pix_ptr[x].b*= WordToByte;  
	
		                    pixColor24 = SMyPixelColor24(pix_ptr[x].r, pix_ptr[x].g, pix_ptr[x].b);

							((SMyPixelColor24*)buffer)[y*width+x].r=pixColor24.r;
							((SMyPixelColor24*)buffer)[y*width+x].g=pixColor24.g;
							((SMyPixelColor24*)buffer)[y*width+x].b=pixColor24.b;
				        }else
					    {
						    pix_ptr[x].r*= WordTo5Bit; pix_ptr[x].g*= WordTo5Bit;
							pix_ptr[x].b*= WordTo5Bit; 
                            
							pixColor16 = SMyPixelColor16(pix_ptr[x].r, pix_ptr[x].g, pix_ptr[x].b);

							((SMyPixelColor16*)buffer)[y*width+x].argb=pixColor16.argb;
			            }
				    }
				}
            } 	

			if (buffer && nBytesAllocated>0 && texDataHeader.ComprMode != MY_TEXDATA_COMPR_SIMPLE_ID)
			{
				if (texDataHeader.ComprMode == MY_TEXDATA_COMPR_RLE_ID)
				// RLE compression
				{	int encodedBytes = rle_encode(
						(unsigned char *)buffer, nBytesAllocated, 
						(unsigned char *)buffer_compressed, nBytesAllocated
						);				
					
					// write RLE header identificator
					id = MY_TEXDATA_RLE_HEADER_ID;
					myWriteToFile(&id, sizeof(id));				

					// write RLE header
					SMyRLEHeader rleHeader;					
					rleHeader.nEncodedBytes = encodedBytes;
					rleHeader.nDecodedBytes = nBytesAllocated;

					myWriteToFile(&rleHeader, sizeof(SMyRLEHeader));

					// write data
					myWriteToFile(buffer_compressed, encodedBytes);
				}
				else
				// none compression
					myWriteToFile(buffer, nBytesAllocated);
			}

			// free buffer
			if (buffer) delete []buffer;
			if (buffer_compressed) delete []buffer_compressed;

        }
        
		//--------------------------------------------------------		
        
        StdUVGen* uvGen = ((BitmapTex *)tex)->GetUVGen();
		if (uvGen) 
        {
			DumpUVGen(uvGen, indentLevel+1);
		}
		
		TextureOutput* texout = ((BitmapTex*)tex)->GetTexout();
		if (texout->GetInvert()) 
        {
			fprintf(pStream,"%s\t%s\n", indent.data(), ID_TEX_INVERT);
		}
		
		fprintf(pStream,"%s\t%s ", indent.data(), ID_BMP_FILTER);
		switch(((BitmapTex*)tex)->GetFilterType()) 
        {
		case FILTER_PYR:  fprintf(pStream,"%s\n", ID_BMP_FILT_PYR); break;
		case FILTER_SAT: fprintf(pStream,"%s\n", ID_BMP_FILT_SAT); break;
		default: fprintf(pStream,"%s\n", ID_BMP_FILT_NONE); break;
		}
	}	
    
	for (int i=0; i<tex->NumSubTexmaps(); i++) 
    {
		DumpTexture(tex->GetSubTexmap(i), tex->ClassID(), i, 1.0f, indentLevel+1);
	}
	
	fprintf(pStream, "%s}\n", indent.data());
}
//-------------------------------------------------------------------
void MAX2MY3D::CalcTextureNumber(Mtl* mtl, int mtlID, int subNo, int *texNumber)
{   int i;
    for (i=0; i<mtl->NumSubTexmaps(); i++) 
    {   Texmap* subTex = mtl->GetSubTexmap(i);
		if (subTex) 
        {	// If it is a standard material we can see if the map is enabled.
			if (mtl->ClassID() == Class_ID(DMTL_CLASS_ID, 0)) 
            {	if (!((StdMat*)mtl)->MapEnabled(i))
					continue;			
			}
			CalcTextureNumber(subTex, mtl->ClassID(), i, texNumber);
		}
	}

	for (i=0; i<mtl->NumSubMtls(); i++) 
    {	Mtl* subMtl = mtl->GetSubMtl(i);
		if (subMtl) 
        	CalcTextureNumber(subMtl, 0, i, texNumber);
	}
}
//-------------------------------------------------------------------
void MAX2MY3D::CalcTextureNumber(Texmap* tex, Class_ID cid, int subNo, int *texNumber)
{   if (texNumber) *texNumber+=1;
    for (int i=0; i<tex->NumSubTexmaps(); i++)     
		CalcTextureNumber(tex->GetSubTexmap(i), tex->ClassID(), i, texNumber);	
}
//-------------------------------------------------------------------
// GeomObject output  
//-------------------------------------------------------------------
void MAX2MY3D::ExportGeomObject(INode* node, int indentLevel)
{
	ObjectState os = node->EvalWorldState(GetStaticFrame());
	if (!os.obj)
		return;
	
	// Targets are actually geomobjects, but we will export them
	// from the camera and light objects, so we skip them here.
	if (os.obj->ClassID() == Class_ID(TARGET_CLASS_ID, 0))
		return;
	
	
	TSTR indent = GetIndent(indentLevel);
	
	ExportNodeHeader(node, ID_GEOMETRY, indentLevel);
	
	ExportNodeTM(node, indentLevel);
	
	if (GetIncludeMesh()) 
	{
		ExportMesh(node, GetStaticFrame(), indentLevel);
	}

	// Node properties (only for geomobjects)
	fprintf(pStream, "%s\t%s %d\n", indent.data(), ID_PROP_MOTIONBLUR, node->MotBlur());
	fprintf(pStream, "%s\t%s %d\n", indent.data(), ID_PROP_CASTSHADOW, node->CastShadows());
	fprintf(pStream, "%s\t%s %d\n", indent.data(), ID_PROP_RECVSHADOW, node->RcvShadows());

	if (GetIncludeAnim()) {
		ExportAnimKeys(node, indentLevel);
	}

	// Export the visibility track
	Control* visCont = node->GetVisController();
	if (visCont) {
		fprintf(pStream, "%s\t%s {\n", indent.data(), ID_VISIBILITY_TRACK);
		DumpFloatKeys(visCont, indentLevel);
		fprintf(pStream, "\t}\n");
	}

	if (GetIncludeMtl()) {
		ExportMaterial(node, indentLevel);
	}

	if (GetIncludeMeshAnim()) {
		ExportAnimMesh(node, indentLevel);
	}
	
	if (GetIncludeIKJoints()) {
		ExportIKJoints(node, indentLevel);
	}
	
	fprintf(pStream,"%s}\n", indent.data());	
}
//-------------------------------------------------------------------
void MAX2MY3D::ExportMaterial(INode* node, int indentLevel)
{	Mtl* mtl = node->GetMtl();
	
	TSTR indent = GetIndent(indentLevel);
	
	// If the node does not have a material, export the wireframe color
	if (mtl) 
    {	int mtlID = mtlList.GetMtlID(mtl);
		if (mtlID >= 0)
        {	fprintf(pStream,"%s\t%s %d\n", indent.data(), ID_MATERIAL_REF, mtlID);
		}
	}
	else 
    {	DWORD c = node->GetWireColor();
		fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_WIRECOLOR,
			Format(Color(GetRValue(c)/255.0f, GetGValue(c)/255.0f, GetBValue(c)/255.0f)));
	}
}

//-------------------------------------------------------------------
// Shape output  
//-------------------------------------------------------------------
void MAX2MY3D::ExportShapeObject(INode* node, int indentLevel)
{
	ExportNodeHeader(node, ID_SHAPE, indentLevel);
	ExportNodeTM(node, indentLevel);
	TimeValue t = GetStaticFrame();
	Matrix3 tm = node->GetObjTMAfterWSM(t);

	TSTR indent = GetIndent(indentLevel);
	
	ObjectState os = node->EvalWorldState(t);
	if (!os.obj || os.obj->SuperClassID()!=SHAPE_CLASS_ID) {
		fprintf(pStream,"%s}\n", indent.data());
		return;
	}
	
	ShapeObject* shape = (ShapeObject*)os.obj;
	PolyShape pShape;
	int numLines;

	// We will output ahspes as a collection of polylines.
	// Each polyline contains collection of line segments.
	shape->MakePolyShape(t, pShape);
	numLines = pShape.numLines;
	
	fprintf(pStream,"%s\t%s %d\n", indent.data(), ID_SHAPE_LINECOUNT, numLines);
	
	for(int poly = 0; poly < numLines; poly++) {
		fprintf(pStream,"%s\t%s %d {\n", indent.data(), ID_SHAPE_LINE, poly);
		DumpPoly(&pShape.lines[poly], tm, indentLevel);
		fprintf(pStream, "%s\t}\n", indent.data());
	}
	
	if (GetIncludeAnim()) {
		ExportAnimKeys(node, indentLevel);
	}
	
	fprintf(pStream,"%s}\n", indent.data());
}
//-------------------------------------------------------------------
void MAX2MY3D::DumpPoly(PolyLine* line, Matrix3 tm, int indentLevel)
{
	int numVerts = line->numPts;
	
	TSTR indent = GetIndent(indentLevel);
	
	if(line->IsClosed()) {
		fprintf(pStream,"%s\t\t%s\n", indent.data(), ID_SHAPE_CLOSED);
	}
	
	fprintf(pStream,"%s\t\t%s %d\n", indent.data(), ID_SHAPE_VERTEXCOUNT, numVerts);
	
	// We differ between true and interpolated knots
	for (int i=0; i<numVerts; i++) {
		PolyPt* pt = &line->pts[i];
		if (pt->flags & POLYPT_KNOT) {
			fprintf(pStream,"%s\t\t%s\t%d\t%s\n", indent.data(), ID_SHAPE_VERTEX_KNOT, i,
				Format(tm * pt->p));
		}
		else {
			fprintf(pStream,"%s\t\t%s\t%d\t%s\n", indent.data(), ID_SHAPE_VERTEX_INTERP,
				i, Format(tm * pt->p));
		}
		
	}
}

//-------------------------------------------------------------------
// Light output  
//-------------------------------------------------------------------
void MAX2MY3D::ExportLightObject(INode* node, int indentLevel)
{
	TimeValue t = GetStaticFrame();
	TSTR indent = GetIndent(indentLevel);

	ExportNodeHeader(node, ID_LIGHT, indentLevel);
	
	ObjectState os = node->EvalWorldState(t);
	if (!os.obj) {
		fprintf(pStream, "%s}\n", indent.data());
		return;
	}
	
	GenLight* light = (GenLight*)os.obj;
	struct LightState ls;
	Interval valid = FOREVER;
	Interval animRange = ip->GetAnimRange();

	light->EvalLightState(t, valid, &ls);

	// This is part os the lightState, but it doesn't
	// make sense to output as an animated setting so
	// we dump it outside of ExportLightSettings()

	fprintf(pStream, "%s\t%s ", indent.data(), ID_LIGHT_TYPE);
	switch(ls.type) {
	case OMNI_LIGHT:  fprintf(pStream, "%s\n", ID_LIGHT_TYPE_OMNI); break;
	case TSPOT_LIGHT: fprintf(pStream, "%s\n", ID_LIGHT_TYPE_TARG);  break;
	case DIR_LIGHT:   fprintf(pStream, "%s\n", ID_LIGHT_TYPE_DIR); break;
	case FSPOT_LIGHT: fprintf(pStream, "%s\n", ID_LIGHT_TYPE_FREE); break;
	}

	ExportNodeTM(node, indentLevel);
	// If we have a target object, export Node TM for the target too.
	INode* target = node->GetTarget();
	if (target) {
		ExportNodeTM(target, indentLevel);
	}

	int shadowMethod = light->GetShadowMethod();
	fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_LIGHT_SHADOWS,
			shadowMethod == LIGHTSHADOW_NONE ? ID_LIGHT_SHAD_OFF :
			shadowMethod == LIGHTSHADOW_MAPPED ? ID_LIGHT_SHAD_MAP :
			ID_LIGHT_SHAD_RAY);

	
	fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_LIGHT_USELIGHT, Format(light->GetUseLight()));
	
	fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_LIGHT_SPOTSHAPE, 
		light->GetSpotShape() == RECT_LIGHT ? ID_LIGHT_SHAPE_RECT : ID_LIGHT_SHAPE_CIRC);

	fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_LIGHT_USEGLOBAL, Format(light->GetUseGlobal()));
	fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_LIGHT_ABSMAPBIAS, Format(light->GetAbsMapBias()));
	fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_LIGHT_OVERSHOOT, Format(light->GetOvershoot()));

	ExclList* el = light->GetExclList();  // DS 8/31/00 . switched to NodeIDTab from NameTab
	if (el->Count()) {
		fprintf(pStream,"%s\t%s {\n", indent.data(), ID_LIGHT_EXCLUSIONLIST);
		fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_NUMEXCLUDED, Format(el->Count()));
		fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_EXCLINCLUDE, Format(el->TestFlag(NT_INCLUDE)));
		fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_EXCL_AFFECT_ILLUM, Format(el->TestFlag(NT_AFFECT_ILLUM)));
		fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_EXCL_AFFECT_SHAD, Format(el->TestFlag(NT_AFFECT_SHADOWCAST)));
		for (int nameid = 0; nameid < el->Count(); nameid++) {
			INode *n = (*el)[nameid];	// DS 8/31/00
			if (n)
				fprintf(pStream,"%s\t\t%s \"%s\"\n", indent.data(), ID_LIGHT_EXCLUDED, n->GetName());
			}
		fprintf(pStream,"%s\t}\n", indent.data());
	}

	// Export light settings for frame 0
	ExportLightSettings(&ls, light, t, indentLevel);

	// Export animated light settings
	if (!valid.InInterval(animRange) && GetIncludeCamLightAnim()) 
    {
		fprintf(pStream,"%s\t%s {\n", indent.data(), ID_LIGHT_ANIMATION);

		TimeValue t = animRange.Start();
		
		while (1) 
        {
			valid = FOREVER; // Extend the validity interval so the camera can shrink it.
			light->EvalLightState(t, valid, &ls);

			t = valid.Start() < animRange.Start() ? animRange.Start() : valid.Start();
			
			// Export the light settings at this frame
			ExportLightSettings(&ls, light, t, indentLevel+1);
			
			if (valid.End() >= animRange.End()) 
            {
				break;
			}
			else 
            {
				t = (valid.End()/GetTicksPerFrame()+GetMeshFrameStep()) * GetTicksPerFrame();
			}
		}

		fprintf(pStream,"%s\t}\n", indent.data());
	}

	// Export animation keys for the light node
	if (GetIncludeAnim()) 
    {
		ExportAnimKeys(node, indentLevel);
		
		// If we have a target, export animation keys for the target too.
		if (target) 
        {
			ExportAnimKeys(target, indentLevel);
		}
	}
	
	fprintf(pStream,"%s}\n", indent.data());
}
//-------------------------------------------------------------------
void MAX2MY3D::ExportLightSettings(LightState* ls, GenLight* light, TimeValue t, int indentLevel)
{
	TSTR indent = GetIndent(indentLevel);

	fprintf(pStream,"%s\t%s {\n", indent.data(), ID_LIGHT_SETTINGS);

	// Frame #
	fprintf(pStream, "%s\t\t%s %d\n",indent.data(), ID_TIMEVALUE, t);

	fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_COLOR, Format(ls->color));
	fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_INTENS, Format(ls->intens));
	fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_ASPECT, Format(ls->aspect));
	
	if (ls->type != OMNI_LIGHT) {
		fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_HOTSPOT, Format(ls->hotsize));
		fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_FALLOFF, Format(ls->fallsize));
	}
	if (ls->type != DIR_LIGHT && ls->useAtten) {
		fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_ATTNSTART, Format(ls->attenStart));
		fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_ATTNEND,	Format(ls->attenEnd));
	}

	fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_TDIST, Format(light->GetTDist(t, FOREVER)));
	fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_MAPBIAS, Format(light->GetMapBias(t, FOREVER)));
	fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_MAPRANGE, Format(light->GetMapRange(t, FOREVER)));
	fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_MAPSIZE, Format(light->GetMapSize(t, FOREVER)));
	fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_LIGHT_RAYBIAS, Format(light->GetRayBias(t, FOREVER)));

	fprintf(pStream,"%s\t}\n", indent.data());
}

//-------------------------------------------------------------------
// Camera output  
//-------------------------------------------------------------------
void MAX2MY3D::ExportCameraObject(INode* node, int indentLevel)
{
	TSTR indent = GetIndent(indentLevel);

	ExportNodeHeader(node, ID_CAMERA, indentLevel);

	INode* target = node->GetTarget();
	if (target) {
		fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_CAMERA_TYPE, ID_CAMERATYPE_TARGET);
	}
	else {
		fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_CAMERA_TYPE, ID_CAMERATYPE_FREE);
	}


	ExportNodeTM(node, indentLevel);
	// If we have a target object, export animation keys for the target too.
	if (target) {
		ExportNodeTM(target, indentLevel);
	}
	
	CameraState cs;
	TimeValue t = GetStaticFrame();
	Interval valid = FOREVER;
	// Get animation range
	Interval animRange = ip->GetAnimRange();
	
	ObjectState os = node->EvalWorldState(t);
	CameraObject *cam = (CameraObject *)os.obj;
	
	cam->EvalCameraState(t,valid,&cs);
	
	ExportCameraSettings(&cs, cam, t, indentLevel);

	// Export animated camera settings
	if (!valid.InInterval(animRange) && GetIncludeCamLightAnim()) {

		fprintf(pStream,"%s\t%s {\n", indent.data(), ID_CAMERA_ANIMATION);

		TimeValue t = animRange.Start();
		
		while (1) {
			valid = FOREVER; // Extend the validity interval so the camera can shrink it.
			cam->EvalCameraState(t,valid,&cs);

			t = valid.Start() < animRange.Start() ? animRange.Start() : valid.Start();
			
			// Export the camera settings at this frame
			ExportCameraSettings(&cs, cam, t, indentLevel+1);
			
			if (valid.End() >= animRange.End()) {
				break;
			}
			else {
				t = (valid.End()/GetTicksPerFrame()+GetMeshFrameStep()) * GetTicksPerFrame();
			}
		}

		fprintf(pStream,"%s\t}\n", indent.data());
	}
	
	// Export animation keys for the light node
	if (GetIncludeAnim()) {
		ExportAnimKeys(node, indentLevel);
		
		// If we have a target, export animation keys for the target too.
		if (target) {
			ExportAnimKeys(target, indentLevel);
		}
	}
	
	fprintf(pStream,"%s}\n", indent.data());
}
//-------------------------------------------------------------------
void MAX2MY3D::ExportCameraSettings(CameraState* cs, CameraObject* cam, TimeValue t, int indentLevel)
{
	TSTR indent = GetIndent(indentLevel);

	fprintf(pStream,"%s\t%s {\n", indent.data(), ID_CAMERA_SETTINGS);

	// Frame #
	fprintf(pStream, "%s\t\t%s %d\n", indent.data(), ID_TIMEVALUE, t);

	if (cs->manualClip) {
		fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_CAMERA_HITHER, Format(cs->hither));
		fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_CAMERA_YON, Format(cs->yon));
	}

	fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_CAMERA_NEAR, Format(cs->nearRange));
	fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_CAMERA_FAR, Format(cs->farRange));
	fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_CAMERA_FOV, Format(cs->fov));
	fprintf(pStream,"%s\t\t%s %s\n", indent.data(), ID_CAMERA_TDIST, Format(cam->GetTDist(t)));

	fprintf(pStream,"%s\t}\n",indent.data());
}

//-------------------------------------------------------------------
// Helper object output  
//-------------------------------------------------------------------
void MAX2MY3D::ExportHelperObject(INode* node, int indentLevel)
{
	TSTR indent = GetIndent(indentLevel);
	ExportNodeHeader(node, ID_HELPER, indentLevel);

	// We don't really know what kind of helper this is, but by exporting
	// the Classname of the helper object, the importer has a chance to
	// identify it.
	Object* helperObj = node->EvalWorldState(0).obj;
	if (helperObj) {
		TSTR className;
		helperObj->GetClassName(className);
		fprintf(pStream,"%s\t%s \"%s\"\n", indent.data(), ID_HELPER_CLASS, className);
	}

	ExportNodeTM(node, indentLevel);

	if (helperObj) {
		TimeValue	t = GetStaticFrame();
		Matrix3		oTM = node->GetObjectTM(t);
		Box3		bbox;

		helperObj->GetDeformBBox(t, bbox, &oTM);

		fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_BOUNDINGBOX_MIN, Format(bbox.pmin));
		fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_BOUNDINGBOX_MAX, Format(bbox.pmax));
	}

	if (GetIncludeAnim()) {
		ExportAnimKeys(node, indentLevel);
	}
	
	fprintf(pStream,"%s}\n", indent.data());
}





//-------------------------------------------------------------------
// Animation output  
//-------------------------------------------------------------------
// If the object is animated, then we will output the entire mesh definition
// for every specified frame. This can result in, dare I say, rather large files.
//
// Many target systems (including MAX itself!) cannot always read back this
// information. If the objects maintains the same number of verices it can be
// imported as a morph target, but if the number of vertices are animated it
// could not be read back in withou special tricks.
// Since the target system for this exporter is unknown, it is up to the
// user (or developer) to make sure that the data conforms with the target system.

void MAX2MY3D::ExportAnimMesh(INode* node, int indentLevel)
{
	ObjectState os = node->EvalWorldState(GetStaticFrame());
	if (!os.obj)
		return;
	
	TSTR indent = GetIndent(indentLevel);
	
	// Get animation range
	Interval animRange = ip->GetAnimRange();
	// Get validity of the object
	Interval objRange = os.obj->ObjectValidity(GetStaticFrame());
	
	// If the animation range is not fully included in the validity
	// interval of the object, then we're animated.
	if (!objRange.InInterval(animRange)) {
		
		fprintf(pStream,"%s\t%s {\n", indent.data(), ID_MESH_ANIMATION);
		
		TimeValue t = animRange.Start();
		
		while (1) {
			// This may seem strange, but the object in the pipeline
			// might not be valid anymore.
			os = node->EvalWorldState(t);
			objRange = os.obj->ObjectValidity(t);
			t = objRange.Start() < animRange.Start() ? animRange.Start() : objRange.Start();
			
			// Export the mesh definition at this frame
			ExportMesh(node, t, indentLevel+1);
			
			if (objRange.End() >= animRange.End()) {
				break;
			}
			else {
				t = (objRange.End()/GetTicksPerFrame()+GetMeshFrameStep()) * GetTicksPerFrame();
			}
		}
		fprintf(pStream,"%s\t}\n", indent.data());
	}
}
//-------------------------------------------------------------------
// Node Header
//-------------------------------------------------------------------
// The Node Header consists of node type (geometry, helper, camera etc.)
// node name and parent node
void MAX2MY3D::ExportNodeHeader(INode* node, TCHAR* type, int indentLevel)
{
	TSTR indent = GetIndent(indentLevel);
	
	// Output node header and object type 
	fprintf(pStream,"%s%s {\n", indent.data(), type);

	// Node name
	fprintf(pStream,"%s\t%s \"%s\"\n", indent.data(), ID_NODE_NAME, FixupName(node->GetName()));
		
	//  If the node is linked, export parent node name
	INode* parent = node->GetParentNode();
	if (parent && !parent->IsRootNode()) 
    {
		fprintf(pStream,"%s\t%s \"%s\"\n", indent.data(), ID_NODE_PARENT, FixupName(parent->GetName()));
	}
}
//-------------------------------------------------------------------
// Node Transformation  
//-------------------------------------------------------------------
void MAX2MY3D::ExportNodeTM(INode* node, int indentLevel)
{
	Matrix3 pivot = node->GetNodeTM(GetStaticFrame());
	TSTR indent = GetIndent(indentLevel);
	
	fprintf(pStream,"%s\t%s {\n", indent.data(), ID_NODE_TM);
	
	// Node name
	// We export the node name together with the nodeTM, because some objects
	// (like a camera or a spotlight) has an additional node (the target).
	// In that case the nodeTM and the targetTM is exported after eachother
	// and the nodeName is how you can tell them apart.
	fprintf(pStream,"%s\t\t%s \"%s\"\n", indent.data(), ID_NODE_NAME, FixupName(node->GetName()));

	// Export TM inheritance flags
	DWORD iFlags = node->GetTMController()->GetInheritanceFlags();
	fprintf(pStream,"%s\t\t%s %d %d %d\n", indent.data(), ID_INHERIT_POS,
		INHERIT_POS_X & iFlags ? 1 : 0,
		INHERIT_POS_Y & iFlags ? 1 : 0,
		INHERIT_POS_Z & iFlags ? 1 : 0);

	fprintf(pStream,"%s\t\t%s %d %d %d\n", indent.data(), ID_INHERIT_ROT,
		INHERIT_ROT_X & iFlags ? 1 : 0,
		INHERIT_ROT_Y & iFlags ? 1 : 0,
		INHERIT_ROT_Z & iFlags ? 1 : 0);

	fprintf(pStream,"%s\t\t%s %d %d %d\n", indent.data(), ID_INHERIT_SCL,
		INHERIT_SCL_X & iFlags ? 1 : 0,
		INHERIT_SCL_Y & iFlags ? 1 : 0,
		INHERIT_SCL_Z & iFlags ? 1 : 0);

	// Dump the full matrix
	DumpMatrix3(&pivot, indentLevel+2);
	
	fprintf(pStream,"%s\t}\n", indent.data());
}
//------------------------------------------------------------------------
// Print out a transformation matrix in different ways.
// Apart from exporting the full matrix we also decompose
// the matrix and export the components.
void MAX2MY3D::DumpMatrix3(Matrix3* m, int indentLevel)
{
	Point3 row;
	TSTR indent = GetIndent(indentLevel);
	
	// Dump the whole Matrix
	row = m->GetRow(0);
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_TM_ROW0, Format(row));
	row = m->GetRow(1);
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_TM_ROW1, Format(row));
	row = m->GetRow(2);
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_TM_ROW2, Format(row));
	row = m->GetRow(3);
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_TM_ROW3, Format(row));
	
	// Decompose the matrix and dump the contents
	AffineParts ap;
	float rotAngle;
	Point3 rotAxis;
	float scaleAxAngle;
	Point3 scaleAxis;
	
	decomp_affine(*m, &ap);

	// Quaternions are dumped as angle axis.
	AngAxisFromQ(ap.q, &rotAngle, rotAxis);
	AngAxisFromQ(ap.u, &scaleAxAngle, scaleAxis);
    
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_TM_POS, Format(ap.t));
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_TM_ROTAXIS, Format(rotAxis));
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_TM_ROTANGLE, Format(rotAngle));
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_TM_SCALE, Format(ap.k));
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_TM_SCALEAXIS, Format(scaleAxis));
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_TM_SCALEAXISANG, Format(scaleAxAngle));

}
//-------------------------------------------------------------------
// Mesh output  
//-------------------------------------------------------------------
void MAX2MY3D::ExportMesh(INode* node, TimeValue t, int indentLevel)
{
    int i;
    Mtl* nodeMtl = node->GetMtl();
    Matrix3 tm = node->GetObjTMAfterWSM(t);
    BOOL negScale = TMNegParity(tm);
    int vx1, vx2, vx3;
    TSTR indent;
    
    indent = GetIndent(indentLevel+1);
    
    ObjectState os = node->EvalWorldState(t);
    if (!os.obj || os.obj->SuperClassID()!=GEOMOBJECT_CLASS_ID) {
        return; // Safety net. This shouldn't happen.
    }
    
    // Order of the vertices. Get 'em counter clockwise if the objects is
    // negatively scaled.
    if (negScale) {
        vx1 = 2;
        vx2 = 1;
        vx3 = 0;
    }
    else {
        vx1 = 0;
        vx2 = 1;
        vx3 = 2;
    }
    
    BOOL needDel;
    TriObject* tri = GetTriObjectFromNode(node, t, needDel);
    if (!tri) {
        return;
    }

    Mesh* mesh = &tri->GetMesh();

    //-------------------------------------------------
    u16 id;

    if (firstMesh)
    {   id = MY_MESH_LIST_ID;        
        myWriteToFile(&id, sizeof(id));
        firstMesh = false;
    }
    id = MY_MESH_HEADER_ID;       
    myWriteToFile(&id, sizeof(id));

    SMyMeshHeader myMeshHeader;
    
    char *meshName = FixupName(node->GetName());

    memset(myMeshHeader.Name, 0, 256);
    if (sizeof(meshName)<256)
        sprintf (myMeshHeader.Name, "%s", meshName);
    else
    {   for (int i=0 ;i<255; i++)
            myMeshHeader.Name[i] = meshName[i];    
        myMeshHeader.Name[255]=0;
    }          

    int mtlID =-1;
    Mtl* mtl = node->GetMtl();
    if (mtl) mtlID = mtlList.GetMtlID(mtl);

    myMeshHeader.MatIndex=mtlID; 

    // calc mesh mapping channels count
    u32 TChannelCnt =0;
    if (mesh->getNumTVerts()&&GetIncludeTextureCoords()) TChannelCnt++;        
    for (int mp = 2; mp < MAX_MESHMAPS-1; mp++) 
    {   if (mesh->mapSupport(mp)&&GetIncludeTextureCoords())             
            if (mesh->getNumMapVerts(mp)) TChannelCnt++;        
    }
    myMeshHeader.TChannelCnt=TChannelCnt;  

    myWriteToFile(&myMeshHeader, sizeof(SMyMeshHeader));
    //-------------------------------------------------       
    
    mesh->buildNormals();
    
    fprintf(pStream, "%s%s {\n",indent.data(),  ID_MESH);
    fprintf(pStream, "%s\t%s %d\n",indent.data(), ID_TIMEVALUE, t);
    fprintf(pStream, "%s\t%s %d\n",indent.data(), ID_MESH_NUMVERTEX, mesh->getNumVerts());
    fprintf(pStream, "%s\t%s %d\n",indent.data(), ID_MESH_NUMFACES, mesh->getNumFaces());
    
	//***********************************************
	// vertices and faces number
	//***********************************************
    u32 vertsNum = mesh->getNumVerts();
	u32 facesNum = mesh->getNumFaces();

	//***********************************************
	// allocate memory for vertices and faces
	//***********************************************
	SMyVertex* my3dVertices = new SMyVertex[vertsNum];
	SMyFace* my3dFaces = new SMyFace[facesNum];

	//***********************************************
    // storing vertices in buffer
	//***********************************************

    fprintf(pStream,"%s\t%s {\n",indent.data(), ID_MESH_VERTEX_LIST);
    for (i=0; i<vertsNum; i++) 
    {
        Point3 v = tm * mesh->verts[i];

        fprintf(pStream, "%s\t\t%s %4d\t%s\n",indent.data(), ID_MESH_VERTEX, i, Format(v));

        my3dVertices[i].Color = SMyColor (255,255,255,255);  // TODO
        // (X,Y,Z) -> (X,Z,Y)
        my3dVertices[i].Coord = SMyVector3 (v.x, v.z, v.y);    
    }
    fprintf(pStream,"%s\t}\n",indent.data());    	


	//***********************************************
    // storing faces in buffer
	//***********************************************

    // To determine visibility of a face, get the vertices in clockwise order.
    // If the objects has a negative scaling, we must compensate for that by
    // taking the vertices counter clockwise
    fprintf(pStream, "%s\t%s {\n",indent.data(), ID_MESH_FACE_LIST);
    for (i=0; i<facesNum; i++) 
    {
        fprintf(pStream,"%s\t\t%s %4d:    A: %4d B: %4d C: %4d AB: %4d BC: %4d CA: %4d",
            indent.data(),
            ID_MESH_FACE, i,
            mesh->faces[i].v[vx1],
            mesh->faces[i].v[vx2],
            mesh->faces[i].v[vx3],
            mesh->faces[i].getEdgeVis(vx1) ? 1 : 0,
            mesh->faces[i].getEdgeVis(vx2) ? 1 : 0,
            mesh->faces[i].getEdgeVis(vx3) ? 1 : 0);

		my3dFaces[i].A = mesh->faces[i].v[vx1];
		my3dFaces[i].B = mesh->faces[i].v[vx2];
		my3dFaces[i].C = mesh->faces[i].v[vx3];

        fprintf(pStream,"\t %s ", ID_MESH_SMOOTHING);
        for (int j=0; j<32; j++) {
            if (mesh->faces[i].smGroup & (1<<j)) {
                if (mesh->faces[i].smGroup>>(j+1)) {
                    fprintf(pStream,"%d,",j+1); // Add extra comma
                } else {
                    fprintf(pStream,"%d ",j+1);
                }
            }
        }
        
        // This is the material ID for the face.
        // Note: If you use this you should make sure that the material ID
        // is not larger than the number of sub materials in the material.
        // The standard approach is to use a modulus function to bring down
        // the material ID.
        fprintf(pStream,"\t%s %d", ID_MESH_MTLID, mesh->faces[i].getMatID());
        
        fprintf(pStream,"\n");
    }
    fprintf(pStream,"%s\t}\n", indent.data()); 

	//***********************************************
	// calculating Face and vertex normals
	//***********************************************
    int  vertA, vertB, vertC;
    Face* f;    
    triangle3d<f32> face;
	vector3d<f32> vA, vB, vC;
	vector3d<f32> nA, nB, nC;
	vector3d<f32> n;

    for (i=0; i<facesNum; i++) 
	{
		f = &mesh->faces[i];  

		vertA = f->getVert(vx1);
		vertB = f->getVert(vx2);
		vertC = f->getVert(vx3);
		
		vA.X = my3dVertices[vertC].Coord.X;
		vA.Y = my3dVertices[vertC].Coord.Y;
		vA.Z = my3dVertices[vertC].Coord.Z;

		vB.X = my3dVertices[vertB].Coord.X;
		vB.Y = my3dVertices[vertB].Coord.Y;
		vB.Z = my3dVertices[vertB].Coord.Z;

		vC.X = my3dVertices[vertA].Coord.X;
		vC.Y = my3dVertices[vertA].Coord.Y;
		vC.Z = my3dVertices[vertA].Coord.Z;

		face.set(vA, vB, vC);

		n=face.getNormal();
		n.normalize();

		nA = nB = nC = n;

		my3dVertices[vertA].Normal = SMyVector3(nA.X, nA.Y, nA.Z);
		my3dVertices[vertB].Normal = SMyVector3(nB.X, nB.Y, nB.Z);
		my3dVertices[vertC].Normal = SMyVector3(nC.X, nC.Y, nC.Z);


	}

	//***********************************************
	// writing vertices and faces in output file
	//***********************************************
    id = MY_VERTS_ID;       
    myWriteToFile(&id, sizeof(id));
    myWriteToFile(&vertsNum, sizeof(vertsNum));	

	for (i=0; i<vertsNum; i++) 
    {
		myWriteToFile(&my3dVertices[i], sizeof(SMyVertex));         
	}

    id = MY_FACES_ID;       
    myWriteToFile(&id, sizeof(id));    
    myWriteToFile(&facesNum, sizeof(facesNum));

	for (i=0; i<facesNum; i++) 
    {
		myWriteToFile(&my3dFaces[i], sizeof(SMyFace));         
	}

	//***********************************************
	// free memory
	//***********************************************	
	delete []my3dVertices;
	delete []my3dFaces;
    
	//***********************************************
    // Export face map texcoords if we have them...
	//***********************************************
    if (GetIncludeTextureCoords() && !CheckForAndExportFaceMap(nodeMtl, mesh, indentLevel+1)) 
    {
        // If not, export standard tverts
        int numTVx = mesh->getNumTVerts();

        fprintf(pStream, "%s\t%s %d\n",indent.data(), ID_MESH_NUMTVERTEX, numTVx);

        if (numTVx) 
        {

            //-------------------------------------------------
            id = MY_TVERTS_ID;       
            myWriteToFile(&id, sizeof(id));
            u32 tvertsNum = numTVx;
            myWriteToFile(&tvertsNum, sizeof(tvertsNum));
            //-------------------------------------------------


            fprintf(pStream,"%s\t%s {\n",indent.data(), ID_MESH_TVERTLIST);
            for (i=0; i<numTVx; i++) 
            {
                UVVert tv = mesh->tVerts[i];
                fprintf(pStream, "%s\t\t%s %d\t%s\n",indent.data(), ID_MESH_TVERT, i, Format(tv));

                //------------------------------------------------
                SMyTVertex currTVert(SMyVector2(tv.x, 1-tv.y));
                myWriteToFile(&currTVert, sizeof(currTVert));
                //------------------------------------------------

            }
            fprintf(pStream,"%s\t}\n",indent.data());
            
            //-------------------------------------------------
            id = MY_TFACES_ID;       
            myWriteToFile(&id, sizeof(id));
            u32 tfacesNum = mesh->getNumFaces();
            myWriteToFile(&tfacesNum, sizeof(tfacesNum));
            //-------------------------------------------------

            fprintf(pStream, "%s\t%s %d\n",indent.data(), ID_MESH_NUMTVFACES, mesh->getNumFaces());
            fprintf(pStream, "%s\t%s {\n",indent.data(), ID_MESH_TFACELIST);
            for (i=0; i<mesh->getNumFaces(); i++) 
            {
                fprintf(pStream,"%s\t\t%s %d\t%d\t%d\t%d\n",
                    indent.data(),
                    ID_MESH_TFACE, i,
                    mesh->tvFace[i].t[vx1],
                    mesh->tvFace[i].t[vx2],
                    mesh->tvFace[i].t[vx3]);

                //------------------------------------------------
                SMyFace currTFace(
                    mesh->tvFace[i].t[vx1], 
                    mesh->tvFace[i].t[vx2], 
                    mesh->tvFace[i].t[vx3]
                    );
                myWriteToFile(&currTFace, sizeof(currTFace));
                //------------------------------------------------

            }
            fprintf(pStream, "%s\t}\n",indent.data());
        }


        // CCJ 3/9/99
        // New for R3 - Additional mapping channels
        for (int mp = 2; mp < MAX_MESHMAPS-1; mp++) {
            if (mesh->mapSupport(mp)) {

                fprintf(pStream, "%s\t%s %d {\n",indent.data(), ID_MESH_MAPPINGCHANNEL, mp);


                int numTVx = mesh->getNumMapVerts(mp);
                fprintf(pStream, "%s\t\t%s %d\n",indent.data(), ID_MESH_NUMTVERTEX, numTVx);

                if (numTVx) 
                {
                    fprintf(pStream,"%s\t\t%s {\n",indent.data(), ID_MESH_TVERTLIST);

                    //-------------------------------------------------
                    id = MY_TVERTS_ID;       
                    myWriteToFile(&id, sizeof(id));
                    u32 tvertsNum = numTVx;
                    myWriteToFile(&tvertsNum, sizeof(tvertsNum));
                    //-------------------------------------------------

                    for (i=0; i<numTVx; i++) 
                    {
                        UVVert tv = mesh->mapVerts(mp)[i];
                        fprintf(pStream, "%s\t\t\t%s %d\t%s\n",indent.data(), ID_MESH_TVERT, i, Format(tv));

                        //------------------------------------------------
                        SMyTVertex currTVert(SMyVector2(tv.x, 1-tv.y));
                        myWriteToFile(&currTVert, sizeof(currTVert));
                        //------------------------------------------------
                    }
                    fprintf(pStream,"%s\t\t}\n",indent.data());
                    
                    //-------------------------------------------------
                    id = MY_TFACES_ID;       
                    myWriteToFile(&id, sizeof(id));
                    u32 tfacesNum = mesh->getNumFaces();
                    myWriteToFile(&tfacesNum, sizeof(tfacesNum));
                    //-------------------------------------------------

                    fprintf(pStream, "%s\t\t%s %d\n",indent.data(), ID_MESH_NUMTVFACES, mesh->getNumFaces());

                    fprintf(pStream, "%s\t\t%s {\n",indent.data(), ID_MESH_TFACELIST);
                    for (i=0; i<mesh->getNumFaces(); i++) 
                    {
                        fprintf(pStream,"%s\t\t\t%s %d\t%d\t%d\t%d\n",
                            indent.data(),
                            ID_MESH_TFACE, i,
                            mesh->mapFaces(mp)[i].t[vx1],
                            mesh->mapFaces(mp)[i].t[vx2],
                            mesh->mapFaces(mp)[i].t[vx3]);

                        //------------------------------------------------
                        SMyFace currTFace(
                            mesh->mapFaces(mp)[i].t[vx1], 
                            mesh->mapFaces(mp)[i].t[vx2], 
                            mesh->mapFaces(mp)[i].t[vx3]
                            );
                        myWriteToFile(&currTFace, sizeof(currTFace));
                        //------------------------------------------------
                    }
                    fprintf(pStream, "%s\t\t}\n",indent.data());
                }
                fprintf(pStream, "%s\t}\n",indent.data()); 
            }
        }
    }

	//***********************************************
    // Export color per vertex info
	//***********************************************
    if (GetIncludeVertexColors()) {
        int numCVx = mesh->numCVerts;

        fprintf(pStream, "%s\t%s %d\n",indent.data(), ID_MESH_NUMCVERTEX, numCVx);
        if (numCVx) {
            fprintf(pStream,"%s\t%s {\n",indent.data(), ID_MESH_CVERTLIST);
            for (i=0; i<numCVx; i++) {
                Point3 vc = mesh->vertCol[i];
                fprintf(pStream, "%s\t\t%s %d\t%s\n",indent.data(), ID_MESH_VERTCOL, i, Format(vc));
            }
            fprintf(pStream,"%s\t}\n",indent.data());
            
            fprintf(pStream, "%s\t%s %d\n",indent.data(), ID_MESH_NUMCVFACES, mesh->getNumFaces());

            fprintf(pStream, "%s\t%s {\n",indent.data(), ID_MESH_CFACELIST);
            for (i=0; i<mesh->getNumFaces(); i++) {
                fprintf(pStream,"%s\t\t%s %d\t%d\t%d\t%d\n",
                    indent.data(),
                    ID_MESH_CFACE, i,
                    mesh->vcFace[i].t[vx1],
                    mesh->vcFace[i].t[vx2],
                    mesh->vcFace[i].t[vx3]);
            }
            fprintf(pStream, "%s\t}\n",indent.data());
        }
    }
    
    if (GetIncludeNormals()) {
        // Export mesh (face + vertex) normals
        fprintf(pStream, "%s\t%s {\n",indent.data(), ID_MESH_NORMALS);
        
        Point3 fn;  // Face normal
        Point3 vn;  // Vertex normal
        int  vert;
        Face* f;
        
        // Face and vertex normals.
        // In MAX a vertex can have more than one normal (but doesn't always have it).
        // This is depending on the face you are accessing the vertex through.
        // To get all information we need to export all three vertex normals
        // for every face.
        for (i=0; i<mesh->getNumFaces(); i++) {
            f = &mesh->faces[i];
            fn = mesh->getFaceNormal(i);
            fprintf(pStream,"%s\t\t%s %d\t%s\n", indent.data(), ID_MESH_FACENORMAL, i, Format(fn));
            
            vert = f->getVert(vx1);
            vn = GetVertexNormal(mesh, i, mesh->getRVertPtr(vert));
            fprintf(pStream,"%s\t\t\t%s %d\t%s\n",indent.data(), ID_MESH_VERTEXNORMAL, vert, Format(vn));
            
            vert = f->getVert(vx2);
            vn = GetVertexNormal(mesh, i, mesh->getRVertPtr(vert));
            fprintf(pStream,"%s\t\t\t%s %d\t%s\n",indent.data(), ID_MESH_VERTEXNORMAL, vert, Format(vn));
            
            vert = f->getVert(vx3);
            vn = GetVertexNormal(mesh, i, mesh->getRVertPtr(vert));
            fprintf(pStream,"%s\t\t\t%s %d\t%s\n",indent.data(), ID_MESH_VERTEXNORMAL, vert, Format(vn));
        }
        
        fprintf(pStream, "%s\t}\n",indent.data());
    }
    
    fprintf(pStream, "%s}\n",indent.data());
    
    if (needDel) 
    {
        delete tri;
    }
}
//-------------------------------------------------------------------
Point3 MAX2MY3D::GetVertexNormal(Mesh* mesh, int faceNo, RVertex* rv)
{
	Face* f = &mesh->faces[faceNo];
	DWORD smGroup = f->smGroup;
	int numNormals;
	Point3 vertexNormal;
	
	// Is normal specified
	// SPCIFIED is not currently used, but may be used in future versions.
	if (rv->rFlags & SPECIFIED_NORMAL) {
		vertexNormal = rv->rn.getNormal();
	}
	// If normal is not specified it's only available if the face belongs
	// to a smoothing group
	else if ((numNormals = rv->rFlags & NORCT_MASK) && smGroup) {
		// If there is only one vertex is found in the rn member.
		if (numNormals == 1) {
			vertexNormal = rv->rn.getNormal();
		}
		else {
			// If two or more vertices are there you need to step through them
			// and find the vertex with the same smoothing group as the current face.
			// You will find multiple normals in the ern member.
			for (int i = 0; i < numNormals; i++) {
				if (rv->ern[i].getSmGroup() & smGroup) {
					vertexNormal = rv->ern[i].getNormal();
				}
			}
		}
	}
	else {
		// Get the normal from the Face if no smoothing groups are there
		vertexNormal = mesh->getFaceNormal(faceNo);
	}
	
	return vertexNormal;
}

//-------------------------------------------------------------------
// Inverse Kinematics (IK) Joint information  
//-------------------------------------------------------------------
void MAX2MY3D::ExportIKJoints(INode* node, int indentLevel)
{
	Control* cont;
	TSTR indent = GetIndent(indentLevel);

	if (node->TestAFlag(A_INODE_IK_TERMINATOR)) 
		fprintf(pStream,"%s\t%s\n", indent.data(), ID_IKTERMINATOR);

	if(node->TestAFlag(A_INODE_IK_POS_PINNED))
		fprintf(pStream,"%s\t%s\n", indent.data(), ID_IKPOS_PINNED);

	if(node->TestAFlag(A_INODE_IK_ROT_PINNED))
		fprintf(pStream,"%s\t%s\n", indent.data(), ID_IKROT_PINNED);

	// Position joint
	cont = node->GetTMController()->GetPositionController();
	if (cont) {
		JointParams* joint = (JointParams*)cont->GetProperty(PROPID_JOINTPARAMS);
		if (joint && !joint->IsDefault()) {
			// Has IK Joints!!!
			fprintf(pStream,"%s\t%s {\n", indent.data(), ID_IKJOINT);
			DumpJointParams(joint, indentLevel+1);
			fprintf(pStream,"%s\t}\n", indent.data());
		}
	}

	// Rotational joint
	cont = node->GetTMController()->GetRotationController();
	if (cont) {
		JointParams* joint = (JointParams*)cont->GetProperty(PROPID_JOINTPARAMS);
		if (joint && !joint->IsDefault()) {
			// Has IK Joints!!!
			fprintf(pStream,"%s\t%s {\n", indent.data(), ID_IKJOINT);
			DumpJointParams(joint, indentLevel+1);
			fprintf(pStream,"%s\t}\n", indent.data());
		}
	}
}
//-------------------------------------------------------------------
void MAX2MY3D::DumpJointParams(JointParams* joint, int indentLevel)
{
	TSTR indent = GetIndent(indentLevel);
	float scale = joint->scale;

	fprintf(pStream,"%s\t%s %s\n", indent.data(), ID_IKTYPE,   joint->Type() == JNT_POS ? ID_IKTYPEPOS : ID_IKTYPEROT);
	fprintf(pStream,"%s\t%s %d\n", indent.data(), ID_IKDOF,    joint->dofs);

	fprintf(pStream,"%s\t%s %d\n", indent.data(), ID_IKXACTIVE,  joint->flags & JNT_XACTIVE  ? 1 : 0);
	fprintf(pStream,"%s\t%s %d\n", indent.data(), ID_IKYACTIVE,  joint->flags & JNT_YACTIVE  ? 1 : 0);
	fprintf(pStream,"%s\t%s %d\n", indent.data(), ID_IKZACTIVE,  joint->flags & JNT_ZACTIVE  ? 1 : 0);

	fprintf(pStream,"%s\t%s %d\n", indent.data(), ID_IKXLIMITED, joint->flags & JNT_XLIMITED ? 1 : 0);
	fprintf(pStream,"%s\t%s %d\n", indent.data(), ID_IKYLIMITED, joint->flags & JNT_YLIMITED ? 1 : 0);
	fprintf(pStream,"%s\t%s %d\n", indent.data(), ID_IKZLIMITED, joint->flags & JNT_ZLIMITED ? 1 : 0);

	fprintf(pStream,"%s\t%s %d\n", indent.data(), ID_IKXEASE,    joint->flags & JNT_XEASE    ? 1 : 0);
	fprintf(pStream,"%s\t%s %d\n", indent.data(), ID_IKYEASE,    joint->flags & JNT_YEASE    ? 1 : 0);
	fprintf(pStream,"%s\t%s %d\n", indent.data(), ID_IKZEASE,    joint->flags & JNT_ZEASE    ? 1 : 0);

	fprintf(pStream,"%s\t%s %d\n", indent.data(), ID_IKLIMITEXACT, joint->flags & JNT_LIMITEXACT ? 1 : 0);

	for (int i=0; i<joint->dofs; i++) {
		fprintf(pStream,"%s\t%s %d %s %s %s\n", indent.data(), ID_IKJOINTINFO, i, Format(joint->min[i]), Format(joint->max[i]), Format(joint->damping[i]));
	}
}

//-------------------------------------------------------------------
//  Material and Texture Export  
//-------------------------------------------------------------------



//-------------------------------------------------------------------
// For a standard material, this will give us the meaning of a map
// givien its submap id.
TCHAR* MAX2MY3D::GetMapID(Class_ID cid, int subNo)
{
	static TCHAR buf[50];
	
	if (cid == Class_ID(0,0)) {
		strcpy(buf, ID_ENVMAP);
	}
	else if (cid == Class_ID(DMTL_CLASS_ID, 0)) {
		switch (subNo) {
		case ID_AM: strcpy(buf, ID_MAP_AMBIENT); break;
		case ID_DI: strcpy(buf, ID_MAP_DIFFUSE); break;
		case ID_SP: strcpy(buf, ID_MAP_SPECULAR); break;
		case ID_SH: strcpy(buf, ID_MAP_SHINE); break;
		case ID_SS: strcpy(buf, ID_MAP_SHINESTRENGTH); break;
		case ID_SI: strcpy(buf, ID_MAP_SELFILLUM); break;
		case ID_OP: strcpy(buf, ID_MAP_OPACITY); break;
		case ID_FI: strcpy(buf, ID_MAP_FILTERCOLOR); break;
		case ID_BU: strcpy(buf, ID_MAP_BUMP); break;
		case ID_RL: strcpy(buf, ID_MAP_REFLECT); break;
		case ID_RR: strcpy(buf, ID_MAP_REFRACT); break;
		}
	}
	else {
		strcpy(buf, ID_MAP_GENERIC);
	}
	
	return buf;
}

//-------------------------------------------------------------------
void MAX2MY3D::DumpUVGen(StdUVGen* uvGen, int indentLevel)
{
	int mapType = uvGen->GetCoordMapping(0);
	TimeValue t = GetStaticFrame();
	TSTR indent = GetIndent(indentLevel+1);
	
	fprintf(pStream,"%s%s ", indent.data(), ID_MAPTYPE);
	
	switch (mapType) {
	case UVMAP_EXPLICIT: fprintf(pStream,"%s\n", ID_MAPTYPE_EXP); break;
	case UVMAP_SPHERE_ENV: fprintf(pStream,"%s\n", ID_MAPTYPE_SPH); break;
	case UVMAP_CYL_ENV:  fprintf(pStream,"%s\n", ID_MAPTYPE_CYL); break;
	case UVMAP_SHRINK_ENV: fprintf(pStream,"%s\n", ID_MAPTYPE_SHR); break;
	case UVMAP_SCREEN_ENV: fprintf(pStream,"%s\n", ID_MAPTYPE_SCR); break;
	}
	
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_U_OFFSET, Format(uvGen->GetUOffs(t)));
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_V_OFFSET, Format(uvGen->GetVOffs(t)));
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_U_TILING, Format(uvGen->GetUScl(t)));
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_V_TILING, Format(uvGen->GetVScl(t)));
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_ANGLE, Format(uvGen->GetAng(t)));
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_BLUR, Format(uvGen->GetBlur(t)));
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_BLUR_OFFSET, Format(uvGen->GetBlurOffs(t)));
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_NOISE_AMT, Format(uvGen->GetNoiseAmt(t)));
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_NOISE_SIZE, Format(uvGen->GetNoiseSize(t)));
	fprintf(pStream,"%s%s %d\n", indent.data(), ID_NOISE_LEVEL, uvGen->GetNoiseLev(t));
	fprintf(pStream,"%s%s %s\n", indent.data(), ID_NOISE_PHASE, Format(uvGen->GetNoisePhs(t)));
}

//-------------------------------------------------------------------
// Face Mapped Material functions 
//-------------------------------------------------------------------
BOOL MAX2MY3D::CheckForAndExportFaceMap(Mtl* mtl, Mesh* mesh, int indentLevel)
{
	if (!mtl || !mesh) {
		return FALSE;
	}
	
	ULONG matreq = mtl->Requirements(-1);
	
	// Are we using face mapping?
	if (!(matreq & MTLREQ_FACEMAP)) {
		return FALSE;
	}
	
	TSTR indent = GetIndent(indentLevel+1);
	
	// OK, we have a FaceMap situation here...
	
	fprintf(pStream, "%s%s {\n", indent.data(), ID_MESH_FACEMAPLIST);
	for (int i=0; i<mesh->getNumFaces(); i++) {
		Point3 tv[3];
		Face* f = &mesh->faces[i];
		make_face_uv(f, tv);
		fprintf(pStream, "%s\t%s %d {\n", indent.data(), ID_MESH_FACEMAP, i);
		fprintf(pStream, "%s\t\t%s\t%d\t%d\t%d\n", indent.data(), ID_MESH_FACEVERT, (int)tv[0].x, (int)tv[0].y, (int)tv[0].z);
		fprintf(pStream, "%s\t\t%s\t%d\t%d\t%d\n", indent.data(), ID_MESH_FACEVERT, (int)tv[1].x, (int)tv[1].y, (int)tv[1].z);
		fprintf(pStream, "%s\t\t%s\t%d\t%d\t%d\n", indent.data(), ID_MESH_FACEVERT, (int)tv[2].x, (int)tv[2].y, (int)tv[2].z);
		fprintf(pStream, "%s\t}\n", indent.data());
	}
	fprintf(pStream, "%s}\n", indent.data());
	
	return TRUE;
}

//-------------------------------------------------------------------
// Misc Utility functions  
//-------------------------------------------------------------------
// Return an indentation string
TSTR MAX2MY3D::GetIndent(int indentLevel)
{
	TSTR indentString = "";
	for (int i=0; i<indentLevel; i++) {
		indentString += "\t";
	}
	
	return indentString;
}
//-------------------------------------------------------------------
// Determine is the node has negative scaling.
// This is used for mirrored objects for example. They have a negative scale factor
// so when calculating the normal we should take the vertices counter clockwise.
// If we don't compensate for this the objects will be 'inverted'.
BOOL MAX2MY3D::TMNegParity(Matrix3 &m)
{
	return (DotProd(CrossProd(m.GetRow(0),m.GetRow(1)),m.GetRow(2))<0.0)?1:0;
}
//-------------------------------------------------------------------
// Return a pointer to a TriObject given an INode or return NULL
// if the node cannot be converted to a TriObject
TriObject* MAX2MY3D::GetTriObjectFromNode(INode *node, TimeValue t, int &deleteIt)
{
	deleteIt = FALSE;
	Object *obj = node->EvalWorldState(t).obj;
	if (obj->CanConvertToType(Class_ID(TRIOBJ_CLASS_ID, 0))) { 
		TriObject *tri = (TriObject *) obj->ConvertToType(t, 
			Class_ID(TRIOBJ_CLASS_ID, 0));
		// Note that the TriObject should only be deleted
		// if the pointer to it is not equal to the object
		// pointer that called ConvertToType()
		if (obj != tri) deleteIt = TRUE;
		return tri;
	}
	else {
		return NULL;
	}
}

//-------------------------------------------------------------------
// From the SDK
// How to calculate UV's for face mapped materials.
static Point3 basic_tva[3] = { 
	Point3(0.0,0.0,0.0),Point3(1.0,0.0,0.0),Point3(1.0,1.0,0.0)
};
static Point3 basic_tvb[3] = { 
	Point3(1.0,1.0,0.0),Point3(0.0,1.0,0.0),Point3(0.0,0.0,0.0)
};
static int nextpt[3] = {1,2,0};
static int prevpt[3] = {2,0,1};

//-------------------------------------------------------------------
void MAX2MY3D::make_face_uv(Face *f, Point3 *tv)
{
	int na,nhid,i;
	Point3 *basetv;
	/* make the invisible edge be 2->0 */
	nhid = 2;
	if (!(f->flags&EDGE_A))  nhid=0;
	else if (!(f->flags&EDGE_B)) nhid = 1;
	else if (!(f->flags&EDGE_C)) nhid = 2;
	na = 2-nhid;
	basetv = (f->v[prevpt[nhid]]<f->v[nhid]) ? basic_tva : basic_tvb; 
	for (i=0; i<3; i++) {  
		tv[i] = basetv[na];
		na = nextpt[na];
	}
}

//-------------------------------------------------------------------
//  String manipulation functions  
//-------------------------------------------------------------------
#define CTL_CHARS  31
#define SINGLE_QUOTE 39
//-------------------------------------------------------------------
// Replace some characters we don't care for.
TCHAR* MAX2MY3D::FixupName(TCHAR* name)
{
	static char buffer[256];
	TCHAR* cPtr;
	
    _tcscpy(buffer, name);
    cPtr = buffer;
	
    while(*cPtr) {
		if (*cPtr == '"')
			*cPtr = SINGLE_QUOTE;
        else if (*cPtr <= CTL_CHARS)
			*cPtr = _T('_');
        cPtr++;
    }
	
	return buffer;
}
//-------------------------------------------------------------------
// International settings in Windows could cause a number to be written
// with a "," instead of a ".".
// To compensate for this we need to convert all , to . in order to make the
// format consistent.
void MAX2MY3D::CommaScan(TCHAR* buf)
{
    for(; *buf; buf++) if (*buf == ',') *buf = '.';
}
//-------------------------------------------------------------------
TSTR MAX2MY3D::Format(int value)
{
	TCHAR buf[50];
	
	sprintf(buf, _T("%d"), value);
	return buf;
}
//-------------------------------------------------------------------
TSTR MAX2MY3D::Format(float value)
{
	TCHAR buf[40];
	
	sprintf(buf, szFmtStr, value);
	CommaScan(buf);
	return TSTR(buf);
}
//-------------------------------------------------------------------
TSTR MAX2MY3D::Format(Point3 value)
{
	TCHAR buf[120];
	TCHAR fmt[120];
	
	sprintf(fmt, "%s\t%s\t%s", szFmtStr, szFmtStr, szFmtStr);
	sprintf(buf, fmt, value.x, value.y, value.z);

	CommaScan(buf);
	return buf;
}
//-------------------------------------------------------------------
TSTR MAX2MY3D::Format(Color value)
{
	TCHAR buf[120];
	TCHAR fmt[120];
	
	sprintf(fmt, "%s\t%s\t%s", szFmtStr, szFmtStr, szFmtStr);
	sprintf(buf, fmt, value.r, value.g, value.b);

	CommaScan(buf);
	return buf;
}
//-------------------------------------------------------------------
TSTR MAX2MY3D::Format(AngAxis value)
{
	TCHAR buf[160];
	TCHAR fmt[160];
	
	sprintf(fmt, "%s\t%s\t%s\t%s", szFmtStr, szFmtStr, szFmtStr, szFmtStr);
	sprintf(buf, fmt, value.axis.x, value.axis.y, value.axis.z, value.angle);

	CommaScan(buf);
	return buf;
}
//-------------------------------------------------------------------
TSTR MAX2MY3D::Format(Quat value)
{
	// A Quat is converted to an AngAxis before output.
	
	Point3 axis;
	float angle;
	AngAxisFromQ(value, &angle, axis);
	
	return Format(AngAxis(axis, angle));
}
//-------------------------------------------------------------------
TSTR MAX2MY3D::Format(ScaleValue value)
{
	TCHAR buf[280];
	
	sprintf(buf, "%s %s", Format(value.s), Format(value.q));
	CommaScan(buf);
	return buf;
}
//-------------------------------------------------------------------
TSTR MAX2MY3D::myFixupName(TSTR badname)
{
	TSTR nodename;
	nodename = badname;
	int qq = nodename.Length();
	while ((nodename[qq--]!='_') && (qq>=0) ) {}
	if (qq>0)
	{
		nodename = nodename.Substr(0,qq+1);
	}
	return nodename;
}

//-------------------------------------------------------------------
TSTR MAX2MY3D::myCutName(TSTR fullname) 
{
	int chPos=-1;

	for (int i=(fullname.Length()-1); i>=0; i--)
	{
		if (fullname[i]=='\\')
		{
			chPos=i;
			break;
		}
	}

	if (chPos>=0) 
		return fullname.Substr(chPos+1, fullname.Length()+chPos+1);
	else
		return "";
}
